//
// Volume Data Visualizer for Google Earth
//
// mainwindow.cpp
//
// Copyright (c) 2012-2014 Shintaro KAWAHARA (kawahara@jamstec.go.jp)
// Japan Agency for Marine-Earth Science and Technology (JAMSTEC)
// http://www.jamstec.go.jp
//
// This program is free software.
// You can redistribute it and/or modify it under the terms of the
// GNU General Public License version 3.0 as published by the Free
// Software Foundation and appearing in the file LICENSE.GPL included
// in the packaging of this file.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include "mainwindow.h"
#include "ui_mainwindow.h"

Grads grads;
Parameter param;
AppParameter appParam;

MainWindow::MainWindow( QWidget *parent ) :
	QMainWindow( parent ), ui( new Ui::MainWindow )
{
	ui->setupUi( this );

	makeToolBar();
	m_ToolBar->setDisabled( true );

	ui->menuExport->setDisabled( true );
	ui->actionExportMovie->setDisabled( true );
	ui->actionSave->setDisabled( true );
	ui->actionLoad->setDisabled( true );
	ui->actionSaveColorTable->setDisabled( true );
	ui->actionLoadColorTable->setDisabled( true );
	ui->horizontalSlider_SetCurrentTime->hide();
	ui->pushButton_StartAnimation->hide();

#ifdef Q_WS_WIN
	QString ffmpegFileName = QCoreApplication::applicationDirPath() + "/ffmpeg.exe";
	QFile ffmpegEXE( ffmpegFileName );
	if ( ffmpegEXE.exists() == true ) {
		ui->actionExportMovie->setDisabled( false );
	}
#else
	QString ffmpegFileName = "/ffmpeg";
	QString pathString = QString( getenv( "PATH" ) );
	QStringList pathList = pathString.split( QRegExp( ":" ), QString::SkipEmptyParts );
	for ( int i = 0; i < pathList.size(); i++ ) {
		if ( QFile::exists( pathList.at(i) + ffmpegFileName ) ) {
			ui->actionExportMovie->setDisabled( false );
			break;
		}
	}
#endif

	m_Time  = new QTime;
	m_InterruptTimer = new QTimer;
	m_IsAnimation  = false;
	m_IsFullScreen = false;
	m_IsSphereView = false;
	m_IsVisibleControlPanel = true;
	m_CurrentDir = QDir::homePath();

	connect( m_InterruptTimer, SIGNAL( timeout() ),
			 this, SLOT( timerEvent() ) );
	connect( ui->pushButton_StartAnimation, SIGNAL( released() ),
			 this, SLOT( toggleAnimationState() ) );
	connect( ui->horizontalSlider_SetCurrentTime, SIGNAL( valueChanged( int ) ),
			 this, SLOT( setCurrentTime( int ) ) );
	connect( this, SIGNAL( changeViewMode( bool ) ),
			 ui->widget_Viewer, SLOT( setViewMode( bool ) ) );

	connect( this, SIGNAL( getSnapShot( QString, int ) ),
			 ui->widget_Viewer, SLOT( renderToPNG( QString, int ) ) );

	// Sets the Current Time as Start Time of Execution
	m_Time->start();
	// Starts the Internal Timer with a Timeout Interval of 10 msec
	m_InterruptTimer->start( 10 );

	// Get Margins of MainLayout to toggle Full Screen Mode and Window Mode
	m_Margins = ui->gridLayout->contentsMargins();

	// Get Initial Size of Viewer
	param.setViewerSize( ui->widget_Viewer->size() );
	m_ViewerControl = new ViewerControl();

	connect( m_ViewerControl, SIGNAL( valueChanged( ValueType ) ),
			 this, SLOT( changeViewerSize() ) );

	m_KMLControl = new KMLControl();
	param.setKMLMaxImageSize(
		KML_MAX_IMAGE_SIZE_X,
		KML_MAX_IMAGE_SIZE_Y,
		KML_MAX_IMAGE_SIZE_Z );

	m_FontControl = new FontControl;

	appParam.initialize();
	ErrorCode err;
	err = appParam.load();

	// Check Topography Files
	QString topoFileName = QCoreApplication::applicationDirPath() + "/ETOPO5.DOS";
	QFile topoFile( topoFileName );
	if ( topoFile.exists() ) {
		appParam.setStateTopo( true );
	}
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent( QEvent *e )
{
	QMainWindow::changeEvent( e );
	switch ( e->type() ) {
    case QEvent::LanguageChange:
		ui->retranslateUi( this );
        break;
    default:
        break;
    }
}

void MainWindow::keyPressEvent( QKeyEvent *event )
{
	switch( event->key() )
	{
		case Qt::Key_F1 :
			toggleFullScreen();
			if ( m_IsFullScreen ) {
				m_ToolBar->hide();
				ui->gridLayout->setContentsMargins( 0, 0, 0, 0 );
				ui->menuBar->hide();
				ui->statusBar->hide();
				ui->tabWidget_ControlPanel->hide();
				if ( grads.getGridSizeT() > 1 ) {
					ui->horizontalSlider_SetCurrentTime->hide();
					ui->pushButton_StartAnimation->hide();
				}
				showFullScreen();
			} else {
				m_ToolBar->show();
				ui->gridLayout->setContentsMargins( m_Margins );
				ui->menuBar->show();
				ui->statusBar->show();
				ui->tabWidget_ControlPanel->show();
				if ( grads.getGridSizeT() > 1 ) {
					ui->horizontalSlider_SetCurrentTime->show();
					ui->pushButton_StartAnimation->show();
				}
				showNormal();
			}
			break;
		case Qt::Key_F2 :
			toggleViewMode();
			break;
		case Qt::Key_F3 :
			toggleStateControlPanel();
			break;
		case Qt::Key_F12 :
			toggleConnectionTypeColorEditor();
			break;
		case Qt::Key_A :
			toggleAnimationState();
			break;
		case Qt::Key_C :
			{
				QString dirPath;

#ifdef Q_WS_WIN
				dirPath = QFileDialog::getExistingDirectory(
					this, tr( "Choose Directory to Save" ), m_CurrentDir,
					QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog );
#else
				dirPath = QFileDialog::getExistingDirectory(
					this, tr( "Choose Directory to Save" ), m_CurrentDir,
					QFileDialog::ShowDirsOnly );
#endif

				if ( !dirPath.isEmpty() ) {
					m_CurrentDir = dirPath;
					emit getSnapShot( dirPath, param.getCurrentStep() );
				}
				break;
			}
		case Qt::Key_Escape :
			{
				QMessageBox::StandardButton reply
					= QMessageBox::question( this, tr( "Message" ),
						tr( "Quit Application?" ),
						QMessageBox::Yes | QMessageBox::No );
				if ( reply == QMessageBox::Yes ) {
					close();
				}
				break;
			}
		default:
			break;
	}
}

void MainWindow::openGradsFile( const QString &filename )
{
	QString msg;
	ErrorCode err = grads.openFile( filename );
	if ( err == GRADS_NO_ERROR ) {
		param.initialize();

		msg.prepend( "File opened : " + filename.toLocal8Bit() );
		statusBar()->showMessage( msg );

		if ( grads.getGridSizeT() > 1 ) {
			ui->horizontalSlider_SetCurrentTime->show();
			ui->horizontalSlider_SetCurrentTime->setMinimum( 0 );
			ui->horizontalSlider_SetCurrentTime->setMaximum( grads.getGridSizeT() - 1 );
			ui->pushButton_StartAnimation->show();
		} else {
			ui->horizontalSlider_SetCurrentTime->hide();
			ui->pushButton_StartAnimation->hide();
		}
		param.setViewerSize( ui->widget_Viewer->size() );

		m_ToolBar->setDisabled( false );

		ui->menuExport->setDisabled( false );
		ui->actionSave->setDisabled( false );
		ui->actionLoad->setDisabled( false );
		ui->actionSaveColorTable->setDisabled( false );
		ui->actionLoadColorTable->setDisabled( false );
		ui->horizontalSlider_SetCurrentTime->setValue( 0 );
		ui->pushButton_StartAnimation->setIcon( QIcon( ":/resources/play.png" ) );
		m_IsAnimation = false;

		m_Action_ToolButton_DefaultView->setChecked( true );
		m_Action_ToolButton_Projection->setIcon( QIcon( ":/resources/perspective.png" ) );

		emit openGradsFileSucceeded();
	} else {
		msg  = "Could not open GrADS file.\n";
		msg += "ERROR: " + getErrString( err );
		QMessageBox::warning( this, tr( "Warning" ),
			msg, QMessageBox::Ok | QMessageBox::Cancel );
		return;
	}
}

void MainWindow::setCurrentTime( int time )
{
    if ( grads.setData( time ) == true ) {
		emit setCurrentTimeSucceeded();
    }
	param.setCurrentStep( ui->horizontalSlider_SetCurrentTime->value() );
}

void MainWindow::timerEvent()
{
	int currentStep = param.getCurrentStep();

	if ( m_IsAnimation ) {
		if ( currentStep > param.getTimeEnd() ) {
			currentStep = param.getTimeBegin();
		}
//		current_step_data_0 = ( int )fmod( current_time * FPS, TIMESTEP_DATA_0 );
//		m_Time->elapsed()
		param.setCurrentStep( currentStep + 1 );
		ui->horizontalSlider_SetCurrentTime->setValue( currentStep );
	}
	if ( currentStep < param.getTimeBegin() ) {
		param.setCurrentStep( param.getTimeBegin() );
		ui->horizontalSlider_SetCurrentTime->setValue( param.getTimeBegin() );
	} else if ( currentStep > param.getTimeEnd() ) {
		param.setCurrentStep( param.getTimeEnd() );
		ui->horizontalSlider_SetCurrentTime->setValue( param.getTimeEnd() );
	}
	emit execTimerEvent( m_Time->elapsed() );
}

void MainWindow::toggleAnimationState()
{
	m_IsAnimation = !m_IsAnimation;
	if ( m_IsAnimation ) {
		ui->pushButton_StartAnimation->setIcon( QIcon( ":/resources/pause.png" ) );
	} else {
		ui->pushButton_StartAnimation->setIcon( QIcon( ":/resources/play.png"  ) );
		m_Time->restart();
	}
}

void MainWindow::toggleViewMode()
{
	m_IsSphereView = !m_IsSphereView;
	if ( m_IsSphereView ) {
		m_ToolBar->setDisabled( true );
		statusBar()->showMessage( "Change Viewer Type to Spherical Mode." );
	} else {
		m_ToolBar->setDisabled( false );
		statusBar()->showMessage( "Change Viewer Type to Box Mode." );
	}
	emit changeViewMode( m_IsSphereView );
}

void MainWindow::toggleProjection()
{
	if ( param.getProjection() == ORTHO ) {
		m_Action_ToolButton_Projection->setIcon(
			QIcon( ":/resources/perspective.png" ) );
	} else {
		m_Action_ToolButton_Projection->setIcon(
			QIcon( ":/resources/ortho.png" ) );
	}
	ui->widget_Viewer->toggleProjection();
}

void MainWindow::toggleStateControlPanel()
{
	m_IsVisibleControlPanel = !m_IsVisibleControlPanel;
	if ( m_IsVisibleControlPanel ) {
		m_ToolBar->show();
		ui->tabWidget_ControlPanel->show();
	} else {
		if ( m_IsFullScreen ) {
			m_ToolBar->hide();
		}
		ui->tabWidget_ControlPanel->hide();
	}

	ui->widget_Viewer->hide(); // refresh viewer size
	ui->widget_Viewer->show(); // refresh viewer size
	param.setViewerSize( ui->widget_Viewer->size() );
}

void MainWindow::toggleConnectionTypeColorEditor()
{
	if ( ui->widget_ColorEditor->getEditor()->getConnectionType()
		== GradientEditor::CurveConnection ) {
		ui->widget_ColorEditor->getEditor()->setConnectionType( GradientEditor::LineConnection );
	} else {
		ui->widget_ColorEditor->getEditor()->setConnectionType( GradientEditor::CurveConnection );
	}
}

void MainWindow::setObjectToCenter()
{
	param.setObjectToCenter();
}

void MainWindow::updateColorTable()
{
//	emit openGradsFileSucceeded();
	ui->widget_ColorEditor->getEditor()->getColorControlPointFromParam();
	ui->widget_ColorEditor->setColorTableToParam();
}

void MainWindow::changeViewerSize()
{
	QSize viewerSize  = ui->widget_Viewer->size();
	QSize diffSize = param.getViewerSize() - viewerSize;
	setFixedSize( size() + diffSize );
}

void MainWindow::makeToolBar()
{
	m_ToolBar = new QToolBar;
	m_ToolBar->setMovable( false );

	QActionGroup *actionGroup;
	actionGroup = new QActionGroup( m_ToolBar );
	m_Action_ToolButton_DefaultView = new QAction(
		QIcon( ":/resources/default.png" ),
		"D", actionGroup );
	m_Action_ToolButton_FrontView = new QAction(
		QIcon( ":/resources/frontview.png" ),
		"F", actionGroup );
	m_Action_ToolButton_LeftSideView = new QAction(
		QIcon( ":/resources/leftsideview.png" ),
		"L", actionGroup );
	m_Action_ToolButton_BottomView = new QAction(
		QIcon( ":/resources/bottomview.png" ),
		"B", actionGroup );
	m_Action_ToolButton_Projection = new QAction(
		QIcon( ":/resources/perspective.png" ),
		"Projection Mode", this );
	m_Action_ToolButton_Centering = new QAction(
		QIcon( ":/resources/centering.png" ),
		"Centering", this );

	m_Action_ToolButton_DefaultView->setToolTip( "Default View" );
	m_Action_ToolButton_FrontView->setToolTip( "Front View" );
	m_Action_ToolButton_LeftSideView->setToolTip( "Left Side View" );
	m_Action_ToolButton_BottomView->setToolTip( "Bottom View" );
	m_Action_ToolButton_DefaultView->setCheckable( true );
	m_Action_ToolButton_FrontView->setCheckable( true );
	m_Action_ToolButton_LeftSideView->setCheckable( true );
	m_Action_ToolButton_BottomView->setCheckable( true );
	m_Action_ToolButton_Projection->setCheckable( false );
	m_Action_ToolButton_Centering->setCheckable( false );
	m_Action_ToolButton_DefaultView->setChecked( true );
	actionGroup->setExclusive( true );
	m_ToolBar->addAction( m_Action_ToolButton_DefaultView );
	m_ToolBar->addAction( m_Action_ToolButton_FrontView );
	m_ToolBar->addAction( m_Action_ToolButton_LeftSideView );
	m_ToolBar->addAction( m_Action_ToolButton_BottomView );
	m_ToolBar->addSeparator();
	m_ToolBar->addAction( m_Action_ToolButton_Projection );
	m_ToolBar->addSeparator();
	m_ToolBar->addAction( m_Action_ToolButton_Centering );

	addToolBar( m_ToolBar );

	connect( m_Action_ToolButton_DefaultView, SIGNAL( activated() ),
		ui->widget_Viewer, SLOT( setViewDirectionDefault() ) );
	connect( m_Action_ToolButton_FrontView, SIGNAL( activated() ),
		ui->widget_Viewer, SLOT( setViewDirectionFront() ) );
	connect( m_Action_ToolButton_LeftSideView, SIGNAL( activated() ),
		ui->widget_Viewer, SLOT( setViewDirectionLeft() ) );
	connect( m_Action_ToolButton_BottomView, SIGNAL( activated() ),
		ui->widget_Viewer, SLOT( setViewDirectionBottom() ) );
	connect( m_Action_ToolButton_Projection, SIGNAL( triggered() ),
		this, SLOT( toggleProjection() ) );
	connect( m_Action_ToolButton_Centering, SIGNAL( triggered() ),
		this, SLOT( setObjectToCenter() ) );
}

void MainWindow::on_actionOpen_triggered()
{
	// Open New GrADS Data Descriptor File
	QString filename;

#ifdef Q_WS_WIN
	filename = QFileDialog::getOpenFileName(
			this, "Open File",
			m_CurrentDir, "Grads Data Descriptor File (*.ctl)",
			0, QFileDialog::DontUseNativeDialog );
#else
	filename = QFileDialog::getOpenFileName(
			this, "Open File",
			m_CurrentDir, "Grads Data Descriptor File (*.ctl)" );
#endif

	if ( !filename.isEmpty() ) {
		m_CurrentDir = QFileInfo( filename ).path();
		openGradsFile( filename );
	}
}

void MainWindow::on_actionExportKML_triggered()
{
	emit exportKML();
}

void MainWindow::on_actionExportSequentialImages_triggered()
{
	QString dirPath;

#ifdef Q_WS_WIN
	dirPath = QFileDialog::getExistingDirectory(
		this, tr( "Choose Directory to Save" ), m_CurrentDir,
		QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog );
#else
	dirPath = QFileDialog::getExistingDirectory(
		this, tr( "Choose Directory to Save" ), m_CurrentDir,
		QFileDialog::ShowDirsOnly );
#endif

	if ( !dirPath.isEmpty() ) {
		m_CurrentDir = dirPath;
		statusBar()->showMessage( "Exporting Sequential Images....." );
		QApplication::setOverrideCursor( Qt::WaitCursor );
		int currentStep = param.getCurrentStep();
		for ( int t = param.getTimeBegin(); t <= param.getTimeEnd(); t++ ) {
			param.setCurrentStep( t );
			QApplication::processEvents( QEventLoop::ExcludeUserInputEvents );
			ui->horizontalSlider_SetCurrentTime->setValue( t );
			QApplication::processEvents( QEventLoop::ExcludeUserInputEvents );
			emit getSnapShot( dirPath, t );
		}
		param.setCurrentStep( currentStep );
		ui->horizontalSlider_SetCurrentTime->setValue( currentStep );
		QApplication::restoreOverrideCursor();
		statusBar()->showMessage( "Finished to Export Sequential Images." );
	}
}

void MainWindow::on_actionExportMovie_triggered()
{
	QString savefile;

#ifdef Q_WS_WIN
	savefile = QFileDialog::getSaveFileName(
			this, "Save as Movie File", m_CurrentDir,
			"Windows Media Video 8 (*.wmv);;All Files(*)",
			0, QFileDialog::DontUseNativeDialog );
#else
	savefile = QFileDialog::getSaveFileName(
			this, "Save as Movie File", m_CurrentDir,
			"Windows Media Video 8 (*.wmv);;All Files(*)" );
#endif

	if ( !savefile.isEmpty() ) {
		m_CurrentDir = QFileInfo( savefile ).path();
		statusBar()->showMessage( "Exporting Movie File....." );

		QDir tmpDir;
		tmpDir.setPath( QFileInfo( savefile ).path() );
		if ( !tmpDir.exists( "_tmpVDVGE" ) ) {
			tmpDir.mkdir( "_tmpVDVGE" );
		}
		tmpDir.cd( "_tmpVDVGE" );

		// generate sequential images
		QApplication::setOverrideCursor( Qt::WaitCursor );
		int currentStep = param.getCurrentStep();
		for ( int t = param.getTimeBegin(); t <= param.getTimeEnd(); t++ ) {
			QApplication::processEvents( QEventLoop::ExcludeUserInputEvents );
			param.setCurrentStep( t );
			ui->horizontalSlider_SetCurrentTime->setValue( t );
			emit getSnapShot( tmpDir.path(), t - param.getTimeBegin() );
		}

		// output movie
		QString filename = QFileInfo( savefile ).fileName();
		QString suffix   = QFileInfo( savefile ).suffix();
		if ( suffix != "wmv" ) {
			filename = filename + ".wmv";
		}
		QString command1 = QString( "cd \"" + tmpDir.path() + "\"");
		QString command2;
#ifdef Q_WS_WIN
		QString appPath = QCoreApplication::applicationDirPath();
		command2 = QString( "\"" + appPath + "/ffmpeg.exe\" -y -r 12 -i " );
		command2 += QString( "./snapshot_%04d.png ../" );
		command2 += filename;
#else
		command2 = QString( "ffmpeg -y -r 12 -i ./snapshot_%04d.png ../" );
		command2 += filename;
#endif
		QString command  = command1 + " && " + command2;
		system( command.toLocal8Bit().constData() );

		QStringList filters;
		filters << "snapshot_*.png";
		tmpDir.setNameFilters( filters );
		QStringList filtered_files = tmpDir.entryList( QDir::Files, QDir::Name );
		QList<QString>::iterator f;
		for ( f = filtered_files.begin(); f != filtered_files.end(); ++f ) {
			QFile tempFile( tmpDir.path() + "/" + (*f) );
			tmpDir.remove( tempFile.fileName() );
		}
		tmpDir.cdUp();
		tmpDir.rmdir( "_tmpVDVGE" );

		param.setCurrentStep( currentStep );
		ui->horizontalSlider_SetCurrentTime->setValue( currentStep );

		statusBar()->showMessage( "Finished to Export Movie File." );

		QApplication::restoreOverrideCursor();
	}
}

void MainWindow::on_actionSave_triggered()
{
	QString filename;

#ifdef Q_WS_WIN
	filename = QFileDialog::getSaveFileName(
			this, "Save File", m_CurrentDir,
			"VDVGE Save File (*.save);;All Files(*)",
			0, QFileDialog::DontUseNativeDialog );
#else
	filename = QFileDialog::getSaveFileName(
			this, "Save File", m_CurrentDir,
			"VDVGE Save File (*.save);;All Files(*)" );
#endif

	if ( !filename.isEmpty() ) {
		QString suffix = QFileInfo( filename ).suffix();
		if ( suffix != "save" ) {
			filename = filename + ".save";
		}
		m_CurrentDir = QFileInfo( filename ).path();
		QApplication::setOverrideCursor( Qt::WaitCursor );
		ui->widget_ColorEditor->getEditor()->setColorControlPointToParam();
		param.save( filename );
		QString msg;
		msg.prepend( "Saved Parameters to : " + filename.toLocal8Bit() );
		statusBar()->showMessage( msg );
		QApplication::restoreOverrideCursor();
	}
}

void MainWindow::on_actionLoad_triggered()
{
	QString filename;

#ifdef Q_WS_WIN
	filename = QFileDialog::getOpenFileName(
			this, "Load File", m_CurrentDir,
			"VDVGE Save File (*.save);;All Files(*)",
			0, QFileDialog::DontUseNativeDialog );
#else
	filename = QFileDialog::getOpenFileName(
			this, "Load File", m_CurrentDir,
			"VDVGE Save File (*.save);;All Files(*)" );
#endif

	if ( !filename.isEmpty() ) {
		QString msg;
		m_CurrentDir = QFileInfo( filename ).path();
		QApplication::setOverrideCursor( Qt::WaitCursor );
		ErrorCode err = param.load( filename );
		if ( err == PARAM_NO_ERROR ) {
			emit openGradsFileSucceeded();
			ui->widget_ColorEditor->getEditor()->getColorControlPointFromParam();
			msg.prepend( "Loaded Parameters from : " + filename.toLocal8Bit() );
			statusBar()->showMessage( msg );
			QApplication::restoreOverrideCursor();
		} else {
			QApplication::restoreOverrideCursor();
			msg.prepend( "Failed to Load Parameters from : " + filename.toLocal8Bit() );
			statusBar()->showMessage( msg );
			msg  = "Syntax Error!\n";
			msg += "ERROR: " + getErrString( err );
			QMessageBox::warning( this, tr( "Warning" ),
				msg, QMessageBox::Ok );
		}
	}
}

void MainWindow::on_actionSaveColorTable_triggered()
{
	QString filename;

#ifdef Q_WS_WIN
	filename = QFileDialog::getSaveFileName(
			this, "Save Color Table", m_CurrentDir,
			"VDVGE Color Table File (*.colortable);;All Files(*)",
			0, QFileDialog::DontUseNativeDialog );
#else
	filename = QFileDialog::getSaveFileName(
			this, "Save Color Table", m_CurrentDir,
			"VDVGE Color Table File (*.colortable);;All Files(*)" );
#endif

	if ( !filename.isEmpty() ) {
		QString suffix = QFileInfo( filename ).suffix();
		if ( suffix != "colortable" ) {
			filename = filename + ".colortable";
		}
		m_CurrentDir = QFileInfo( filename ).path();
		QApplication::setOverrideCursor( Qt::WaitCursor );
		ui->widget_ColorEditor->getEditor()->setColorControlPointToParam();
		param.saveColorTable( filename );
		QString msg;
		msg.prepend( "Saved ColorTable to : " + filename.toLocal8Bit() );
		statusBar()->showMessage( msg );
		QApplication::restoreOverrideCursor();
	}
}

void MainWindow::on_actionLoadColorTable_triggered()
{
	QString filename;

#ifdef Q_WS_WIN
	filename = QFileDialog::getOpenFileName(
			this, "Load Color Table", m_CurrentDir,
			"VDVGE Color Table File (*.colortable);;All Files(*)",
			0, QFileDialog::DontUseNativeDialog );
#else
	filename = QFileDialog::getOpenFileName(
			this, "Load Color Table", m_CurrentDir,
			"VDVGE Color Table File (*.colortable);;All Files(*)" );
#endif

	if ( !filename.isEmpty() ) {
		QString msg;
		m_CurrentDir = QFileInfo( filename ).path();
		QApplication::setOverrideCursor( Qt::WaitCursor );
		ErrorCode err = param.loadColorTable( filename );
		if ( err == PARAM_COLORTABLE_NO_ERROR ) {
			emit openGradsFileSucceeded();
			ui->widget_ColorEditor->getEditor()->getColorControlPointFromParam();
			msg.prepend( "Loaded ColorTable from : " + filename.toLocal8Bit() );
			statusBar()->showMessage( msg );
			QApplication::restoreOverrideCursor();
		} else {
			QApplication::restoreOverrideCursor();
			msg.prepend( "Failed to Load ColorTable from : " + filename.toLocal8Bit() );
			statusBar()->showMessage( msg );
			msg  = "Syntax Error!\n";
			msg += "ERROR: " + getErrString( err );
			QMessageBox::warning( this, tr( "Warning" ),
				msg, QMessageBox::Ok );
		}
	}
}

void MainWindow::on_actionChangeViewerSize_triggered()
{
	m_ViewerControl->popup();
}

void MainWindow::on_actionKMLImageSize_triggered()
{
	m_KMLControl->popup();
}

void MainWindow::on_actionFont_triggered()
{
	m_FontControl->popup();
}

void MainWindow::on_actionExit_triggered()
{
	QMessageBox::StandardButton reply
		= QMessageBox::question( this, tr( "Message" ),
			tr( "Quit Application?" ),
			QMessageBox::Yes | QMessageBox::No );
	if ( reply == QMessageBox::Yes ) {
		close();
	}
}
